/*
    Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of Auto-Off digitalSTROM Server add on.

    Auto-Off is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Auto-Off is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

Ext.namespace('AutoOff', 'AutoOff.config');

AutoOff.VERSION = '0.8.0';
AutoOff.lang = 'de_DE';


AutoOff.SetTrigger = function(record, oldVal)
{
    var zoneParam = 'ZoneID=' + record.get('ZoneID') + ';';
    var offParam = 'offDelay=' + record.get('offDelay');

    var failMsg = _("Could not add auto off configuration for room") +
        ' ' + record.get('name');

    Ext.Ajax.request(
    {
        url: '/json/event/raise',
        params:
        {
            name: "auto-off_set",
            parameter: zoneParam + offParam
        },
        disableCaching: true,
        method: 'GET',
        success: function(response, opts)
        {
            try
            {
                // if dSS is not running stunnel still gives us an HTTP OK
                if (response.responseText.length === 0)
                {
                    record.set('offDelay', oldVal);
                    record.commit();
                    Ext.Msg.alert(_("Error"), failMsg + ': ' +
                        _("dSS is not running"));
                    return;
                }

                var json = Ext.decode(response.responseText);
                if (json.ok !== true)
                {
                    record.set('offDelay', oldVal);
                    record.commit();
                    Ext.Msg.alert(_("Error"), failMsg + ': ' + json.message);
                }
            }
            catch (error)
            {
                record.set('offDelay', oldVal);
                record.commit();
                Ext.Msg.alert(_("Error"), failMsg + ': ' + error);
            }
        },
        failure: function(response, opts)
        {
            try
            {
                var json = Ext.decode(response.responseText);
                if (json.ok !== true)
                {
                    failMsg = failMsg + ': ' + json.message;
                }
            }
            catch (err) {}

            record.set('offDelay', oldVal);
            record.commit();
            Ext.Msg.alert(_("Error"), failMsg);
        }
    });
};

AutoOff.MainPanel = Ext.extend(Ext.grid.EditorGridPanel,
{
    initComponent: function()
    {
        var queryString = '/scripts/auto-off/settings/*(ZoneID,offDelay)';
        this.triggerStore = new Ext.data.JsonStore(
        {
            url: '/json/property/query?query=' + queryString,
            root: 'result.settings',
            idProperty: 'ZoneID',
            fields: [ { name: 'offDelay', type: 'int' } ],
            successProperty: 'ok'
        });

        var zoneQueryString =
                 '/apartment/zones/*(ZoneID,name)/devices/*(present)';
        this.zoneStore = new Ext.data.JsonStore(
        {
            url: '/json/property/query?query=' + zoneQueryString,
            root: 'result.zones',
            idProperty: 'ZoneID',
            fields:
            [
                'name',
                'ZoneID',
                'devices',
                {
                    name: 'offDelay',
                    type: 'int',
                    defaultValue: 0
                }
            ],
            successProperty: 'ok'
        });

        var editTriggerRenderer = function(value, metaData, record, rowIndex,
                                      colIndex, store)
        {
            metaData.css = 'icon-edit-trigger';
            return String.format('<img class="padding-img" src="{0}"/>',
                                 Ext.BLANK_IMAGE_URL);
        };

        var deleteTriggerRenderer = function(value, metaData, record, rowIndex,
                                      colIndex, store)
        {
            if (record.get('offDelay') < 1)
            {
                metaData.css = 'disabled-icon-delete-trigger';
            }
            else
            {
                metaData.css = 'icon-delete-trigger';
            }
            return String.format('<img class="padding-img" src="{0}"/>',
                                 Ext.BLANK_IMAGE_URL);
        };

        var tplZone = new Ext.XTemplate(
            '<span class="x-green-text">',
                '<tpl if="name !== undefined && name.length &gt; 0">',
                  '{name}',
                '</tpl>',
                '<tpl if="name === undefined || name.length === 0">',
                  _("Zone") + ' #', '{ZoneID}',
                '</tpl>',
            '</span>'
        );
        tplZone.compile();

        var tplConfig = new Ext.XTemplate(
                 '<div class="x-green-text">',
                '</div>'
        );
        tplConfig.compile();

        var tplMinutes = new Ext.XTemplate(
            '<span class="x-green-text">',
                '<tpl if="offDelay &gt; 0">',
                  _("Automatically turn light off after") + ':&nbsp;',
                '</tpl>',
                '<tpl if="offDelay === 0 || offDelay === &quot;&quot;">',
                  _("Auto off not configured"),
                '</tpl>',
                '<tpl if="offDelay === 1">',
                  '<b>',
                  '{offDelay}',
                  '&nbsp;' + _("minute", "minutes", 1),
                  '</b>',
                '</tpl>',
                '<tpl if="offDelay &gt; 1">',
                  '<b>',
                  '{offDelay}',
                  '&nbsp;' + _("minute", "minutes", 2),
                  '</b>',
                '</tpl>',
            '</span>'
        );
        tplMinutes.compile();

        Ext.apply(this,
        {
            store: this.zoneStore,
            scope: this,
            enableHDMenu: false,
            enableColumnHide: false,
            stripeRows: true,
            trackMouseOver: true,
            bodyBorder: false,
            clicksToEdit: 1,
            enableColumnMove: false,
            sm: new Ext.grid.RowSelectionModel({singleSelect:true}),
            width: 900,
            viewConfig:
            {
                forceFit: true
            },
            columns:
            [
                {
                    header: _("Room"),
                    dataIndex: 'name',
                    sortable: true,
                    hideable: false,
                    xtype: 'templatecolumn',
                    tpl: tplZone
                },
                {
                    header: _("Auto-off configuration status"),
                    sortable: true,
                    hideable: false,
                    xtype: 'templatecolumn',
                    dataIndex: 'offDelay',
                    fixed: true,
                    width: 350,
                    tpl: tplMinutes
                },
                {
                    header: '&#160;',
                    menuDisabled: true,
                    dataIndex: 'offDelay',
                    fixed: true,
                    width: 48,
                    editable: true,
                    renderer: editTriggerRenderer,
                    editor: new Ext.form.NumberField(
                    {
                        allowBlank: false,
                        allowNegative: false,
                        allowDecimals: false
                    })
                },
                {
                    header: '&#160;',
                    menuDisabled: true,
                    dataIndex: 'offDelay',
                    fixed: true,
                    width: 24,
                    renderer: deleteTriggerRenderer
                }

            ],
            bbar:
            {
                xtype: 'toolbar',
                items:
                [
                    {
                        xtype: 'tbtext',
                        cls:'x-black-text x-text-align-left x-text-cut',
                        text: _("Version") + ': ' + AutoOff.VERSION
                    },
                    {
                        xtype: 'tbseparator',
                        width: 8
                    },
                    '->',
                    {
                        xtype: 'button',
                        ref: 'help',
                        iconCls: 'icon-help',
                        text: _("Help"),
                        scope: this,
                        handler: this.showHelp.createDelegate(this)
                    },
                    {
                        xtype: 'button',
                        ref: 'refresh',
                        iconCls: 'icon-refresh-view',
                        text: _("Refresh view"),
                        scope: this,
                        handler: this.loadData.createDelegate(this)
                    }
                ]
            }
        });

        AutoOff.MainPanel.superclass.initComponent.apply(this, arguments);

        this.triggerStore.on('load',
            function(store, records, options)
            {
                Ext.each(records, function(rec)
                    {
                        var zrec = this.store.getById(rec.id);
                        if (zrec !== undefined)
                        {

                            zrec.set('offDelay', rec.get('offDelay'));
                            zrec.commit();
                        }
                    },
                    this);

                this.filterZoneStore();
                // grid will get updated
                this.store.resumeEvents();
            },
            this);


        // settings node does not yet exist
        this.triggerStore.on('exception',
            function()
            {
                this.filterZoneStore();
                // grid will get updated
                this.store.resumeEvents();
            },
            this);

        this.on('afteredit',
            function(e)
            {
                e.record.commit();
                AutoOff.SetTrigger(e.record, e.value);
            },
            this);

        this.on('cellclick',
                function(grid, rowIndex, columnIndex, event)
                {
                    if (columnIndex === 3)
                    {
                        var record = this.getStore().getAt(rowIndex);
                        var old = record.get('offDelay');
                        if (old > 0)
                        {
                            record.set('offDelay', 0);
                            record.commit();
                            AutoOff.SetTrigger(record, old);
                        }
                    }
                },
                this);

        this.on('afterrender', this.loadData.createDelegate(this), this);
    },
    loadData: function()
    {
        // we only want to show the already filtered store, so suspend events
        // when loading and resume events after filtering
        this.zoneStore.suspendEvents(true);
        this.zoneStore.load(
        {
            callback: function()
            {
                this.triggerStore.load();
            },
            scope: this
        });
    },
    filterZoneStore: function()
    {
        this.store.filterBy(
            function(rec)
            {
                if (rec.get('ZoneID') === 0)
                {
                    return false;
                }

                var devices = rec.get('devices');
                if (devices === undefined) {
                    return false;
                }

                if (devices.length === 0)
                {
                    return false;
                }

                var i;
                for (i = 0; i < devices.length; i++)
                {
                    if (devices[i].present === true)
                    {
                        return true;
                    }
                }

                return false;
            },
            this);
    },
    showHelp: function()
    {
        var win = new Ext.Window(
        {
            title: _("Help"),
            width: 480,
            height: 320,
            layout: 'fit',
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    autoScroll: true,
                    autoLoad: 'locale/' + dss.staticDataModel.activeLanguage + '/help.html'
                }
            ],
            buttons:
            [
                {
                    text: _("Close"),
                    scope: this,
                    handler: function() { win.close(); }
                }
            ]
        });
        win.show(this);
    }
});

Ext.reg('mainpanel', AutoOff.MainPanel);

AutoOff.Main = function()
{
    this.viewport = new Ext.Viewport(
    {
        hideBorders: true,
        layout: 'border',
        items:
            [
                {
                    xtype: 'panel',
                    region: 'north',
                    layout: 'hbox',
                    layoutConfig:
                    {
                       align : 'stretch',
                        pack  : 'start'
                    },

                    height: 75,
                    items:
                    [
                        {
                            xtype: 'box',
                            cls: 'switch-anim',
                            width: 128
                        },
                        {
                            xtype: 'box',
                            cls: 'auto-off-logo',
                            width: 337
                        },
                        {
                            xtype: 'box',
                            flex: 1
                        },
                        {
                            xtype: 'box',
                            width: 110,
                            cls: 'dS-logo'
                        }

                    ]
                },
                {
                    region: 'center',
                    xtype: 'mainpanel'
                }
            ]
    });
};

// security features are currently being implemented in the dSS but are
// not yet complete, for now we use a function with a default login so that
// that we are able to access the property tree
AutoOff.dSSLogin = function()
{
    Ext.Ajax.request({
        url: '/json/system/login',
        params:
        {
            user: 'dssadmin',
            password: 'dssadmin'
        },
        scope: this,
        method: "GET",
        disableCaching: true,
        success: function(response, options) {
            try {
                // dSS not running
                if (response.responseText.length === 0) {
                    Ext.Msg.alert(_("Error"),
                        _("No response from dSS." +
                          " Check if the dSS is running and reload this page " +
                          "again!"));
                    return;

                }

                var jsonData = Ext.util.JSON.decode(response.responseText);
                if (jsonData.ok !== true) {
                    Ext.Msg.alert(_("Error"),
                        _("Could not log in to dSS, can not continue") + ': ' +
                        json.message);
                    return;
                }

               AutoOff.Main();

            } catch (error) {
                Ext.Msg.alert(_("Error"),
                              _("Could not log in to dSS, can not continue") +
                              ': ' + error.message);
            }
        },
        failure: function(response, options) {
            Ext.Msg.alert(_("Error"),
                              _("Could not log in to dSS, can not continue"));
        }
    });
};

Ext.onReady(
    function()
    {
        dss.buildUpLang(['locale/{languageSuffix}/auto-off.po']);

        Ext.get('start').remove();


        AutoOff.dSSLogin();
    }
);
